import json
import re

import lxml.html
import requests


def by_url(article_url: str):
    html = html_by_url(article_url)
    return scrape_html(html)


def html_by_url(article_url: str):
    return requests.get(article_url).text


def scrape_html(article_html: str):
    doc = lxml.html.fromstring(article_html)

    ld_content = doc.xpath('string(//script[@type="application/ld+json"]/text())')
    ld = json.loads(ld_content)
    ld_by_type = {ld_entry['@type']: ld_entry for ld_entry in ld}
    news_ld = ld_by_type['NewsArticle']

    settings = json.loads(doc.xpath('string(//script[@type="application/settings+json"]/text())'))
    info = settings['editorial']['info']

    text_node_selector = \
        'main .word-wrap > p,'  \
        'main .word-wrap > h3, ' \
        'main .word-wrap > ul > li, ' \
        'main .word-wrap > ol > li'
    text_nodes = doc.cssselect(text_node_selector)
    text = re.sub(r'\n+', '\n', '\n'.join([node.text_content() for node in text_nodes])).strip()

    return {
        'url': doc.xpath('string(//link[@rel="canonical"]/@href)'),
        'id': info['article_id'],
        'channel': info['channel'],
        'subchannel': info['subchannel'],
        'headline': {
            'main': info['headline'],
            'social': info['headline_social']
        },
        'intro': info['intro'],
        'text': text,
        'topics': info['topics'],
        'author': settings['editorial']['author'],
        'comments_enabled': settings['editorial']['attributes']['is_comments_enabled'],
        'date_created': news_ld['dateCreated'],
        'date_modified': news_ld['dateModified'],
        'date_published': news_ld['datePublished'],
        'breadcrumbs': [breadcrumb['item']['name'] for breadcrumb in ld_by_type['BreadcrumbList']['itemListElement']],
    }




from collections import defaultdict

import requests

from spiegel_scraper.constants import TALK_ENDPOINT_URL


def by_article_id(article_id: str, page_size_limit: int = 1000):
    query = """
        query GetComments($assetId: ID!, $cursor: Cursor, $limit: Int) { 
          asset(id: $assetId) {
            comments(deep: true, query: {sortBy: CREATED_AT, sortOrder: ASC, limit: $limit, cursor: $cursor}) {
              endCursor
              hasNextPage
              nodes {
                id
                parent {id}
                body
                tags {
                  tag {
                    name
                    created_at
                  }
                  assigned_by {
                    id
                    username
                    role
                  }
                }
                user {
                  id
                  username
                  role
                }
                status
                created_at
                updated_at
                editing {
                  edited
                }
                richTextBody
                highlights {
                  start
                  end
                }
              }
            }
          }
        }
    """

    # fetch all comments into flat array
    comments = []
    cursor = None
    while True:
        resp = requests.post(TALK_ENDPOINT_URL, json={'query': query, 'variables': {
            'assetId': article_id,
            'cursor': cursor,
            'limit': page_size_limit,
        }})

        data = resp.json()['data']
        comments_data = data['asset']['comments']
        comments.extend(comments_data['nodes'])
        cursor = comments_data['endCursor']
        if not comments_data['hasNextPage']:
            break

    # build nested reply structure
    root_comments = []
    replies_by_id = defaultdict(list)
    for comment in comments:
        comment['replies'] = replies_by_id[comment['id']]
        parent_ref = comment.pop('parent')
        if parent_ref is None:
            root_comments.append(comment)
        else:
            replies_by_id[parent_ref['id']].append(comment)

    return root_comments



import datetime as dt
import re

import dateparser
import requests
import lxml.html
import tldextract


def by_date(date: dt.date):
    html = html_by_date(date)
    return scrape_html(html)


def html_by_date(date: dt.date):
    archive_url = f'https://www.spiegel.de/nachrichtenarchiv/artikel-{date.strftime("%d.%m.%Y")}.html'
    resp = requests.get(archive_url)
    return resp.text


def scrape_html(html: str):
    doc = lxml.html.fromstring(html)
    url = doc.xpath('string(//link[@rel="canonical"]/@href)')
    date_string = re.search(r'/artikel-(\d{2}\.\d{2}\.\d{4})\.html$', url)[1]
    timestamp = dt.datetime.strptime(date_string, '%d.%m.%Y')

    articles = []
    for article in doc.xpath('//article'):
        url = article.xpath('string(.//a/@href)')

        # omit external layouts such as Bento and Manager Magazin
        if tldextract.extract(url).registered_domain != 'spiegel.de':
            continue

        articles.append({
            'url': url,
            'headline': article.xpath('string(.//a/@title)'),
            'is_paid': article.xpath('boolean(.//svg/title[text()="Icon: Spiegel Plus"])'),
            'date_published': parse_date(article.xpath('string(./footer/span[1])'), relative_base=timestamp),
            'channel': article.xpath('string(./footer/span[3])'),
        })

    return articles


def parse_date(date_string: str, relative_base: dt.datetime):
    return dateparser.parse(
        date_string=date_string,
        languages=['de'],
        settings={
            'RELATIVE_BASE': relative_base,
        },
    )
